# -*- coding: utf-8 -*-
"""notify module"""

__author__  = "Adrien DELLE CAVE"
__license__ = """
    Copyright (C) 2017-2018  fjord-technologies

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA..
"""

import logging

from datetime import datetime
from dwho.classes.notifiers import DWhoPushNotifications
from dwho.classes.modules import DWhoModuleBase, MODULES
from dwho.config import DWHO_SHARED
from httpdis.ext.httpdis_json import HttpReqErrJson
from sonicprobe.libs import xys
from sonicprobe.libs.moresynchro import RWLock


LOG = logging.getLogger('xtrargon.modules.notify')

_DATE_FORMAT = '%Y-%m-%d_%H-%M-%S'


class XtrargonNotifyModule(DWhoModuleBase):
    MODULE_NAME = 'notify'

    LOCK        = RWLock()

    def safe_init(self, options):
        self.server_id     = self.config['general']['server_id']
        self.lock_timeout  = self.config['general']['lock_timeout']
        self.notifications = DWhoPushNotifications(config_path = self.config['general']['notifiers_path'])


    BACKUP_SCHEMA = xys.load("""
    uid!-: !!str
    client_id!-: !!str
    backup!-: !!str
    type!-: !!str
    partial*: !!bool False
    status!-: !~~enum(processing,fail,success)
    backup_started*-: !!str
    backup_ended*-: !!str
    hostname!-: !!str
    errors*: [ !!str ]
    """)

    def backup(self, request):
        args = request.payload_params()

        if not xys.validate(args, self.BACKUP_SCHEMA):
            raise HttpReqErrJson(415, "invalid arguments for command")

        if not self.LOCK.acquire_write(self.lock_timeout):
            raise HttpReqErrJson(503, "unable to take LOCK for writing after %s seconds" % self.lock_timeout)

        try:
            if args['type'] not in ('full', 'incr'):
                return

            xvars = args.copy()
            xvars['total_seconds'] = None

            if args.get('backup_started') and args.get('backup_ended'):
                started                = datetime.strptime(args['backup_started'], _DATE_FORMAT)
                ended                  = datetime.strptime(args['backup_ended'], _DATE_FORMAT)
                xvars['total_seconds'] = (ended - started).total_seconds()

            DWHO_SHARED.set("xtrargon://%s" % args['uid'], 'state', args)

            self.notifications(xvars)

            return xvars
        except HttpReqErrJson:
            raise
        except Exception, e:
            LOG.exception(e)
            raise HttpReqErrJson(503, repr(e))
        finally:
            self.LOCK.release()


if __name__ != "__main__":
    def _start():
        MODULES.register(XtrargonNotifyModule())
    _start()
